def test_rethink_server(rethink_server):
    assert rethink_server.check_server_up()
    assert rethink_server.conn.db == 'test'


FIXTURE_TABLES = [('tbl_foo', 'code'), ('tbl_bar', 'id')]

# Lots of tests needed here!

def test_rethink_empty_db(rethink_empty_db):
    pass

def test_foo_1(rethink_empty_db):
    assert 1

def test_foo_2(rethink_empty_db):
    assert 2

def test_foo_3(rethink_empty_db):
    assert 3


# coding: utf-8

from __future__ import absolute_import, division, print_function, unicode_literals


def test_connection(s3_bucket):
    client, bucket_name = s3_bucket
    bucket = client.Bucket(bucket_name)
    assert bucket is not None



import pytest


def test_mongo_server(mongo_server):
    assert mongo_server.check_server_up()
    assert mongo_server.delete
    mongo_server.api.db.test.insert({'a': 'b', 'c': 'd'})
    assert mongo_server.api.db.test.find_one({'a': 'b'}, {'_id': False}) == {'a': 'b', 'c': 'd'}


@pytest.mark.parametrize('count', range(3))
def test_mongo_server_multi(count, mongo_server):
    coll = mongo_server.api.some_database.some_collection
    assert coll.count() == 0
    coll.insert({'a': 'b'})
    assert coll.count() == 1



'''
Created on 25 Apr 2012

@author: eeaston

'''
from __future__ import absolute_import
import socket

import pytest

from pytest_server_fixtures import CONFIG
from pytest_fixture_config import requires_config

from .base import TestServer


def _redis_server(request):
    """ Does the redis server work, this is used within different scoped
        fixtures.
    """
    test_server = RedisTestServer()
    request.addfinalizer(lambda p=test_server: p.teardown())
    test_server.start()
    return test_server


@requires_config(CONFIG, ['redis_executable'])
@pytest.fixture(scope='function')
def redis_server(request):
    """ Function-scoped Redis server in a local thread.

        Attributes
        ----------
        api: (``redis.Redis``)   Redis client API connected to this server
        .. also inherits all attributes from the `workspace` fixture
    """
    return _redis_server(request)


@requires_config(CONFIG, ['redis_executable'])
@pytest.fixture(scope='session')
def redis_server_sess(request):
    """ Same as redis_server fixture, scoped for test session
    """
    return _redis_server(request)


class RedisTestServer(TestServer):
    """This will look for 'redis_executable' in configuration and use as the
    redis-server to run.
    """
    port_seed = 65532

    def __init__(self, db=0, **kwargs):
        global redis
        try:
            import redis
        except ImportError:
            pytest.skip('redis not installed, skipping test')
        self.db = db
        super(RedisTestServer, self).__init__(**kwargs)
        self.api = redis.Redis(host=self.hostname, port=self.port, db=self.db)

    @property
    def run_cmd(self):
        return [CONFIG.redis_executable, '-']

    @property
    def run_stdin(self):
        # these don't work on redis-server 2.2.12 on ubuntu precise32.
        # Need to come up with a way of detecting redis server version
        # and setting the appropriate config variant.
        #
        # zset-max-ziplist-value 64
        # zset-max-ziplist-entries 128
        # vm-enabled no
        # hash-max-zipmap-entries 512
        # hash-max-zipmap-value 64
        # list-max-ziplist-entries 512
        # list-max-ziplist-value 64
        # set-max-intset-entries 512
        # activerehashing yes

        cfg = ("""
        daemonize no
        port %(port)d
        bind %(hostname)s
        timeout 0
        loglevel notice
        logfile %(workspace)s/redis.log
        databases 1
        maxmemory %(maxmemory)s
        maxmemory-policy noeviction
        appendonly no
        slowlog-log-slower-than -1
        slowlog-max-len 1024
        """ % {
            'port': self.port,
            'hostname': socket.gethostbyname(self.hostname),
            'workspace': self.workspace,
            'maxmemory': "2G",
        }).strip()
        # print "---------------\ncfg:\n%s\n---------------" % cfg
        return cfg

    def check_server_up(self):
        """ Ping the server
        """
        try:
            print("pinging Redis at %s:%s db %s" % (
                self.hostname, self.port, self.db
            ))
            return redis.Redis(
                host=self.hostname,
                port=self.port,
                db=self.db
            ).ping()
        except redis.ConnectionError as e:
            print("server not up yet (%s)" % e)
            return False



import codecs
import os
import signal
import subprocess
import time
from tempfile import mkdtemp

import pytest

from pytest_shutil.workspace import Workspace
from pytest_fixture_config import yield_requires_config

from pytest_server_fixtures import CONFIG


@yield_requires_config(CONFIG, ['xvfb_executable'])
@pytest.yield_fixture(scope='function')
def xvfb_server():
    """ Function-scoped Xvfb (X-Windows Virtual Frame Buffer) in a local thread.
    """
    test_server = XvfbServer()
    yield test_server
    test_server.close()


@yield_requires_config(CONFIG, ['xvfb_executable'])
@pytest.yield_fixture(scope='session')
def xvfb_server_sess():
    """ Session-scoped Xvfb (X-Windows Virtual Frame Buffer) in a local thread.
    """
    test_server = XvfbServer()
    yield test_server
    test_server.close()



# TODO: make this a TestServer, clean up print statements for proper logging
class XvfbServer(object):
    # see https://github.com/revnode/xvfb-run/blob/master/xvfb-run
    xvfb_command = 'Xvfb'
    xvfb_args = '-screen 0 1024x768x24 -nolisten tcp -reset -terminate'.split()
    display, authfile, process, fbmem = None, None, None, None

    def __init__(self):
        tmpdir = mkdtemp(prefix='XvfbServer.', dir=Workspace.get_base_tempdir())
        for servernum in range(os.getpid(), 65536):
            if os.path.exists('/tmp/.X{0}-lock'.format(servernum)):
                continue
            self.display = ':' + str(servernum)
            self.authfile = os.path.join(tmpdir, 'Xauthority.' + self.display)
            mcookie = codecs.encode(os.urandom(16), "hex_codec")
            subprocess.check_call(['xauth', '-f', self.authfile, 'add', self.display, '.', mcookie])
            errfile = os.path.join(tmpdir, 'Xvfb.' + self.display + '.err')
            with open(errfile, 'w') as f:  # use a file instead of a pipe to simplify polling
                p = subprocess.Popen([self.xvfb_command, self.display, '-fbdir', tmpdir] + self.xvfb_args,
                                     stderr=f, env=dict(os.environ, XAUTHORITY=self.authfile))
                self.fbmem = os.path.join(tmpdir, 'Xvfb_screen0')
                while not os.path.exists(self.fbmem):
                    if p.poll() is not None:
                        break
                    time.sleep(0.1)
                else:
                    p.poll()
                if p.returncode is not None:
                    with open(errfile) as f:
                        err = f.read()
                    if 'Server is already active for display' in err:
                        continue
                    else:
                        raise RuntimeError('Failed to start Xvfb', p.returncode, err)
            print('Xvfb started in ' + tmpdir)  # for debugging
            self.process = p
            # If we terminate abnormally, ensure the Xvfb server is cleaned up after us.
            self._cleanup_script = subprocess.Popen("""
while kill -0 {0} 2>/dev/null; do sleep 1; done
kill -INT {1}
while kill -0 {1} 2>/dev/null; do sleep 1; done
""".format(os.getpid(), p.pid), shell=True)
            break
        else:
            raise RuntimeError('Unable to find a free server number to start Xvfb')

    def close(self):
        if self.process is not None:
            if self.process.poll() is None:
                self.process.send_signal(signal.SIGINT)
                self.process.wait()
            self._cleanup_script.kill()
        self.process, self._cleanup_script = None, None

    def __enter__(self):
        return self

    def __exit__(self, *_args):
        self.close()



import socket
import os

from pytest_fixture_config import Config


class FixtureConfig(Config):
    __slots__ = ('java_executable', 'jenkins_url', 'jenkins_war', 'minio_executable', 'mongo_bin', 'redis_executable',
                 'rethink_executable', 'httpd_executable', 'httpd_modules', 'fixture_hostname',
                 'xvfb_executable', 'disable_proxy')

# Default values for system resource locations - patch this to change defaults
DEFAULT_SERVER_FIXTURES_HOSTNAME = socket.gethostname()
DEFAULT_SERVER_FIXTURES_DISABLE_HTTP_PROXY = True
DEFAULT_SERVER_FIXTURES_JAVA = "java"
DEFAULT_SERVER_FIXTURES_JENKINS_URL = 'http://acmejenkins.example.com'
DEFAULT_SERVER_FIXTURES_JENKINS_WAR = '/usr/share/jenkins/jenkins.war'
DEFAULT_SERVER_FIXTURES_MINIO = '/usr/local/bin/minio'
DEFAULT_SERVER_FIXTURES_MONGO_BIN = '/usr/bin'
DEFAULT_SERVER_FIXTURES_REDIS = "/usr/bin/redis-server"
DEFAULT_SERVER_FIXTURES_RETHINK = "/usr/bin/rethinkdb"
DEFAULT_SERVER_FIXTURES_HTTPD = "/usr/sbin/apache2"
DEFAULT_SERVER_FIXTURES_HTTPD_MODULES = "/usr/lib/apache2/modules"
DEFAULT_SERVER_FIXTURES_XVFB = "/usr/bin/Xvfb"

# Global config for finding system resources.
CONFIG = FixtureConfig(
    # Not using localhost here in case we are being used in a cluster-type job
    fixture_hostname=os.getenv('SERVER_FIXTURES_HOSTNAME', DEFAULT_SERVER_FIXTURES_HOSTNAME),
    disable_proxy=os.getenv('SERVER_FIXTURES_DISABLE_HTTP_PROXY', DEFAULT_SERVER_FIXTURES_DISABLE_HTTP_PROXY),
    java_executable=os.getenv('SERVER_FIXTURES_JAVA', DEFAULT_SERVER_FIXTURES_JAVA),
    jenkins_war=os.getenv('SERVER_FIXTURES_JENKINS_WAR', DEFAULT_SERVER_FIXTURES_JENKINS_WAR),
    minio_executable=os.getenv('SERVER_FIXTURES_MINIO', DEFAULT_SERVER_FIXTURES_MINIO),
    mongo_bin=os.getenv('SERVER_FIXTURES_MONGO_BIN', DEFAULT_SERVER_FIXTURES_MONGO_BIN),
    redis_executable=os.getenv('SERVER_FIXTURES_REDIS', DEFAULT_SERVER_FIXTURES_REDIS),
    rethink_executable=os.getenv('SERVER_FIXTURES_RETHINK', DEFAULT_SERVER_FIXTURES_RETHINK),
    httpd_executable=os.getenv('SERVER_FIXTURES_HTTPD', DEFAULT_SERVER_FIXTURES_HTTPD),
    httpd_modules=os.getenv('SERVER_FIXTURES_HTTPD_MODULES', DEFAULT_SERVER_FIXTURES_HTTPD_MODULES),
    xvfb_executable=os.getenv('SERVER_FIXTURES_XVFB', DEFAULT_SERVER_FIXTURES_XVFB),
)



# coding: utf-8
"""
Pytest fixtures to launch a minio S3 server and get a bucket for it.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import uuid
from collections import namedtuple
import logging
import os

import pytest
from future.utils import text_type
from pytest_fixture_config import requires_config

from . import CONFIG
from .http import HTTPTestServer

log = logging.getLogger(__name__)

def _s3_server(request):
    server = MinioServer()
    server.start()
    request.addfinalizer(server.teardown)
    return server


@requires_config(CONFIG, ['minio_executable'])
@pytest.fixture(scope="session")
def s3_server(request):
    """
    Creates a session-scoped temporary S3 server using the 'minio' tool.

    The primary method on the server object is `s3_server.get_s3_client()`, which returns a boto3 `Resource`
    (`boto3.resource('s3', ...)`)
    """
    return _s3_server(request)

BucketInfo = namedtuple('BucketInfo', ['client', 'name'])
# Minio is a little too slow to start for each function call
# Start it once per session and get a new bucket for each function instead.
@pytest.fixture(scope="function")
def s3_bucket(s3_server):  # pylint: disable=redefined-outer-name
    """
    Creates a function-scoped s3 bucket,
    returning a BucketInfo namedtuple with `s3_bucket.client` and `s3_bucket.name` fields
    """
    client = s3_server.get_s3_client()
    bucket_name = text_type(uuid.uuid4())
    client.create_bucket(Bucket=bucket_name)
    return BucketInfo(client, bucket_name)


class MinioServer(HTTPTestServer):
    random_port = True
    aws_access_key_id = "MINIO_TEST_ACCESS"
    aws_secret_access_key = "MINIO_TEST_SECRET"

    def __init__(self, workspace=None, delete=None, preserve_sys_path=False, **kwargs):
        env = kwargs.get('env', os.environ.copy())
        env.update({"MINIO_ACCESS_KEY": self.aws_access_key_id, "MINIO_SECRET_KEY": self.aws_secret_access_key})
        kwargs['env'] = env
        super(MinioServer, self).__init__(workspace=workspace, delete=delete, preserve_sys_path=preserve_sys_path, **kwargs)

    def get_s3_client(self):
        # Region name and signature are to satisfy minio
        import boto3
        import botocore.client
        s3 = boto3.resource(
            's3',
            endpoint_url=self.boto_endpoint_url,
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            region_name='us-east-1',
            config=botocore.client.Config(signature_version='s3v4'),
        )
        return s3

    @property
    def datadir(self):
        return self.workspace / 'minio-db'

    @property
    def boto_endpoint_url(self):
        return self.uri

    def pre_setup(self):
        self.datadir.mkdir()  # pylint: disable=no-value-for-parameter

    @property
    def run_cmd(self):
        cmdargs = [
            CONFIG.minio_executable,
            "server",
            "--address",
            ":{}".format(self.port),
            text_type(self.datadir),
        ]
        return cmdargs


import os
import tempfile
import shutil
import subprocess
import time
import errno
import logging
import getpass

import pytest

from pytest_server_fixtures import CONFIG
from pytest_fixture_config import yield_requires_config

from .base import TestServer

log = logging.getLogger(__name__)


def _mongo_server():
    """ This does the actual work - there are several versions of this used
        with different scopes.
    """
    test_server = MongoTestServer()
    try:
        test_server.start()
        yield test_server
    finally:
        test_server.teardown()


@yield_requires_config(CONFIG, ['mongo_bin'])
@pytest.yield_fixture(scope='function')
def mongo_server():
    """ Function-scoped MongoDB server started in a local thread.
        This also provides a temp workspace.
        We tear down, and cleanup mongos at the end of the test.

        For completeness, we tidy up any outstanding mongo temp directories
        at the start and end of each test session

        Attributes
        ----------
        api (`pymongo.MongoClient`)  : PyMongo Client API connected to this server
        .. also inherits all attributes from the `workspace` fixture
    """
    for server in _mongo_server():
        yield server


@yield_requires_config(CONFIG, ['mongo_bin'])
@pytest.yield_fixture(scope='session')
def mongo_server_sess():
    """ Same as mongo_server fixture, scoped as session instead.
    """
    for server in _mongo_server():
        yield server


@yield_requires_config(CONFIG, ['mongo_bin'])
@pytest.yield_fixture(scope='class')
def mongo_server_cls(request):
    """ Same as mongo_server fixture, scoped for test classes.
    """
    for server in _mongo_server():
        request.cls.mongo_server = server
        yield server


@yield_requires_config(CONFIG, ['mongo_bin'])
@pytest.yield_fixture(scope='module')
def mongo_server_module():
    """ Same as mongo_server fixture, scoped for test modules.
    """
    for server in _mongo_server():
        yield server


class MongoTestServer(TestServer):
    # Use a random port for Mongo, as we run tests in parallel
    random_port = True

    def __init__(self, **kwargs):
        base_dir = self.get_base_dir()
        if not os.path.exists(base_dir):
            try:
                os.makedirs(base_dir)
            except OSError as exc:  # Python >2.5
                if exc.errno == errno.EEXIST and os.path.isdir(base_dir):
                    pass
                else:
                    raise
        mongod_dir = tempfile.mkdtemp(dir=base_dir)
        super(MongoTestServer, self).__init__(workspace=mongod_dir, delete=True, **kwargs)

    @staticmethod
    def get_base_dir():
        candidate_dir = os.environ.get('WORKSPACE', None)
        if not candidate_dir or not os.path.exists(candidate_dir):
            candidate_dir = os.environ.get('TMPDIR', '/tmp')
        return os.path.join(candidate_dir, getpass.getuser(), 'mongo')

    @property
    def run_cmd(self):
        return ['%s/mongod' % CONFIG.mongo_bin,
                '--bind_ip=%s' % self.hostname,
                '--port=%s' % self.port,
                '--dbpath=%s' % self.workspace,
                '--nounixsocket',
                '--syncdelay', '0',
                '--nohttpinterface',
                '--nojournal',
                '--quiet',
                '--storageEngine=ephemeralForTest'
                ]

    def check_server_up(self):
        """Test connection to the server."""
        import pymongo
        from pymongo.errors import AutoReconnect, ConnectionFailure

        log.info("Connecting to Mongo at %s:%s" % (self.hostname, self.port))
        try:
            self.api = pymongo.MongoClient(self.hostname, self.port,
                                           serverselectiontimeoutms=200)
            self.api.database_names()
            # Configure the client with default timeouts in case the server goes slow
            self.api = pymongo.MongoClient(self.hostname, self.port)
            return True
        except (AutoReconnect, ConnectionFailure) as e:
            pass
        return False

    def kill(self):
        """ We override kill, as the parent kill does way too much.  We are single process
            and simply want to kill the underlying mongod process. """
        if self.server:
            try:
                self.server.exit = True
                self.server.p.kill()
                self.server.p.wait()
                i = 0
                while self.check_server_up():
                    time.sleep(0.1)
                    if i % 10 == 0:
                        log.info("Waiting for MongoServer.kill()")
            except OSError:
                pass
            self.dead = True

    @staticmethod
    def cleanup_all():
        """Helper method which will ensure that there are no running mongos
        on the current host, in the current workspace"""
        base = MongoTestServer.get_base_dir()
        if not os.path.isdir(base):
            return

        log.info("======================================")
        log.info("Cleaning up previous sessions under " + base)
        log.info("======================================")

        for mongo in os.listdir(base):
            if mongo.startswith('tmp'):
                mongo = os.path.join(base, mongo)
                log.info("Previous session: " + mongo)
                lock = os.path.join(mongo, 'mongod.lock')
                if os.path.exists(lock):
                    log.info("Lock file found: " + lock)
                    p = subprocess.Popen(["/usr/sbin/lsof", "-Fp", "--", lock], stdout=subprocess.PIPE)
                    (out, _) = p.communicate()
                    if out:
                        pid = out[1:].strip()
                        log.info("Owned by pid: " + pid + " killing...")
                        p = subprocess.Popen(["kill -9 %s" % pid], shell=True)
                        p.communicate()
                log.info("Removing: " + mongo)
                shutil.rmtree(mongo, True)


# Cleanup any old mongo sessions for this workspace when run in Jenkins
# We do this here rather than doing:
#    request.cached_setup(MongoTestServer.kill_all, scope='session')
# as with pytest-xidst, the per-session setups appear to be run for each worker

# We don't do this for users (who don't have the WORKSPACE env variable set)
# as they may legitimately be running a test suite more than once.

# TODO: check that with the latest py.test this is still the case, work has
#       been done to improve fixtures with xdist
if 'WORKSPACE' in os.environ:
    MongoTestServer.cleanup_all()



# coding: utf-8

from __future__ import absolute_import, division, print_function, unicode_literals

import os
import logging
import subprocess

import errno
import pytest
from six import text_type

from .base import TestServer

log = logging.getLogger(__name__)


@pytest.fixture(scope='session')
def postgres_server_sess(request):
    """A session-scoped Postgres Database fixture"""
    return _postgres_server(request)


def _postgres_server(request):
    server = PostgresServer()
    server.start()
    request.addfinalizer(server.teardown)
    return server


class PostgresServer(TestServer):
    """
    Exposes a server.connect() method returning a raw psycopg2 connection.
    Also exposes a server.connection_config property returning a dict with connection parameters
    """
    random_port = True

    def __init__(self, database_name="integration", skip_on_missing_postgres=False, **kwargs):
        self.database_name = database_name
        # TODO make skip configurable with a pytest flag
        self._fail = pytest.skip if skip_on_missing_postgres else pytest.exit
        super(PostgresServer, self).__init__(workspace=None, delete=True, preserve_sys_path=False, **kwargs)

    def kill(self, retries=5):
        if hasattr(self, 'pid'):
            try:
                os.kill(self.pid, self.kill_signal)
            except OSError as e:
                if e.errno == errno.ESRCH:  # "No such process"
                    pass
                else:
                    raise

    def pre_setup(self):
        """
        Find postgres server binary
        Set up connection parameters
        """
        (self.workspace / 'db').mkdir()  # pylint: disable=no-value-for-parameter

        try:
            self.pg_bin = subprocess.check_output(["pg_config", "--bindir"]).decode('utf-8').rstrip()
        except OSError as e:
            msg = "Failed to get pg_config --bindir: " + text_type(e)
            print(msg)
            self._fail(msg)
        initdb_path = self.pg_bin + '/initdb'
        if not os.path.exists(initdb_path):
            msg = "Unable to find pg binary specified by pg_config: {} is not a file".format(initdb_path)
            print(msg)
            self._fail(msg)
        try:
            subprocess.check_call([initdb_path, str(self.workspace / 'db')])
        except OSError as e:
            msg = "Failed to launch postgres: " + text_type(e)
            print(msg)
            self._fail(msg)

    @property
    def connection_config(self):
        return {
            u'host': u'localhost',
            u'user': os.environ[u'USER'],
            u'port': self.port,
            u'database': self.database_name
        }

    @property
    def run_cmd(self):
        cmd = [
            self.pg_bin + '/postgres',
            '-F',
            '-k', str(self.workspace / 'db'),
            '-D', str(self.workspace / 'db'),
            '-p', str(self.port),
            '-c', "log_min_messages=FATAL"
        ]  # yapf: disable
        return cmd

    def check_server_up(self):
        from psycopg2 import OperationalError
        try:
            print("Connecting to Postgres at localhost:{}".format(self.port))
            with self.connect('postgres') as conn:
                conn.set_session(autocommit=True)
                with conn.cursor() as cursor:
                    cursor.execute("CREATE DATABASE " + self.database_name)
            self.connection = self.connect(self.database_name)
            with open(self.workspace / 'db' / 'postmaster.pid', 'r') as f:
                self.pid = int(f.readline().rstrip())
            return True
        except OperationalError as e:
            print("Could not connect to test postgres: {}".format(e))
        return False

    def connect(self, database=None):
        import psycopg2
        cfg = self.connection_config
        if database is not None:
            cfg[u'database'] = database
        return psycopg2.connect(**cfg)



from __future__ import print_function

import os
import socket
import logging
import time

import pytest
import requests
from contextlib import contextmanager
from six.moves import http_client

from pytest_shutil.env import unset_env
from pytest_server_fixtures import CONFIG
from .base import TestServer


log = logging.getLogger(__name__)


@pytest.yield_fixture
def simple_http_test_server():
    """ Function-scoped py.test fixture to serve up a directory via HTTP.
    """
    with SimpleHTTPTestServer() as s:
        s.start()
        yield s


class HTTPTestServer(TestServer):
    # Bind to all sockets when creating the web-server, for selenium tests
    hostname = '0.0.0.0'

    def __init__(self, uri=None, **kwargs):
        self._uri = uri
        super(HTTPTestServer, self).__init__(**kwargs)

    @property
    def uri(self):
        if self._uri:
            return self._uri
        return "http://%s:%s" % (self.hostname, self.port)

    @contextmanager
    def handle_proxy(self):
        if CONFIG.disable_proxy:
            with unset_env(['http_proxy', 'https_proxy', 'HTTP_PROXY', ' HTTPS_PROXY']):
                yield
        else:
            yield

    def check_server_up(self):
        """ Check the server is up by polling self.uri
        """
        try:
            log.debug('accessing URL: {0}'.format(self.uri))
            with self.handle_proxy():
                resp = requests.get(self.uri)
            acceptable_codes = (200, 403)  # 403 server probably running in secure mode...
            log.debug('Querying %s received response code %s' % (self.uri, resp.status_code))
            return resp.status_code in acceptable_codes
        except requests.ConnectionError as e:
            log.debug("Server not up yet (%s).." % e)
            return False

    def get(self, path, as_json=False, attempts=25):
        """ Queries the server using requests.GET and returns the response object. 
        
        Parameters
        ----------
        path :  `str`
            Path to the resource, relative to 'http://hostname:port/'
        as_json :  `bool`
            Returns the json object if True. Defaults to False.
        attempts: `int`
            This function will retry up to `attempts` times on connection errors, to handle 
            the server still waking up. Defaults to 25.
        """
        e = None
        for i in range(attempts):
            try:
                with self.handle_proxy():
                    returned = requests.get('http://%s:%d/%s' % (self.hostname, self.port, path))
                return returned.json() if as_json else returned
            except (http_client.BadStatusLine, requests.ConnectionError) as e:
                time.sleep(int(i) / 10)
                pass
        raise e

    def post(self, path, data=None, attempts=25, as_json=False):
        """ Posts data to the server using requests.POST and returns the response object. 
        
        Parameters
        ----------
        path :  `str`
            Path to the resource, relative to 'http://hostname:port/'
        as_json :  `bool`
            Returns the json response if True. Defaults to False.
        attempts: `int`
            This function will retry up to `attempts` times on connection errors, to handle 
            the server still waking up. Defaults to 25.
        """
        e = None
        for i in range(attempts):
            try:
                with self.handle_proxy():
                    returned = requests.post('http://%s:%d/%s' % (self.hostname, self.port, path), data=data)
                return returned.json() if as_json else returned
            except (http_client.BadStatusLine, requests.ConnectionError) as e:
                time.sleep(int(i) / 10)
                pass
        raise e


class SimpleHTTPTestServer(HTTPTestServer):
    """A Simple HTTP test server that serves up a folder of files over the web."""

    def __init__(self, workspace=None, delete=None, **kwargs):
        kwargs.pop("hostname", None)  # User can't set the hostname it is always 0.0.0.0
        # If we don't pass hostname="0.0.0.0" to our superclass's initialiser then the cleanup
        # code in kill won't work correctly. We don't set self.hostname however as we want our
        # uri property to still be correct.
        super(SimpleHTTPTestServer, self).__init__(workspace=workspace, delete=delete, hostname="0.0.0.0", **kwargs)
        self.cwd = self.document_root

    @property
    def uri(self):
        if self._uri:
            return self._uri
        return "http://%s:%s" % (socket.gethostname(), self.port)

    @property
    def run_cmd(self):
        return ["python", "-m", "SimpleHTTPServer", str(self.port)]

    @property
    def document_root(self):
        """This is the folder of files served up by this SimpleHTTPServer"""
        file_dir = os.path.join(str(self.workspace), "files")
        if not os.path.exists(file_dir):
            os.mkdir(file_dir)
        return file_dir



import sys
import os
sys.path.append(os.path.dirname(os.path.dirname(__file__)))

from setuptools import setup, find_packages
from common_setup import common_setup

classifiers = [
    'License :: OSI Approved :: MIT License',
    'Development Status :: 5 - Production/Stable',
    'Topic :: Software Development :: Libraries',
    'Topic :: Software Development :: Testing',
    'Topic :: Utilities',
    'Intended Audience :: Developers',
    'Operating System :: POSIX',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
]

install_requires = ['pytest',
                    'pytest-shutil',
                    'pytest-fixture-config',
                    'six',
                    'future',
                    'requests',
                    ]

extras_require = {
    'jenkins':  ["python-jenkins"],
    'mongodb':  ["pymongo"],
    'postgres': ["psycopg2"],
    'rethinkdb':  ["rethinkdb"],
    'redis':  ["redis"],
    's3': ["boto3"]
}

tests_require = [
                 'mock',
                 'psutil',
                 ]

entry_points = {
    'pytest11': [
        'httpd_server = pytest_server_fixtures.httpd',
        'jenkins_server = pytest_server_fixtures.jenkins',
        'mongodb_server = pytest_server_fixtures.mongo',
        'postgres_server = pytest_server_fixtures.postgres',
        'redis_server = pytest_server_fixtures.redis',
        'rethinkdb_server = pytest_server_fixtures.rethink',
        'xvfb_server = pytest_server_fixtures.xvfb',
        's3 = pytest_server_fixtures.s3',
    ]
}

if __name__ == '__main__':
    kwargs = common_setup('pytest_server_fixtures')
    kwargs.update(dict(
        name='pytest-server-fixtures',
        description='Extensible server fixures for py.test',
        author='Edward Easton',
        author_email='eeaston@gmail.com',
        classifiers=classifiers,
        install_requires=install_requires,
        extras_require=extras_require,
        tests_require=tests_require,
        packages=find_packages(exclude='tests'),
        entry_points=entry_points,
    ))
    setup(**kwargs)
